--[[ simpleMinimap ]]--

local smmVersion,smmSubVersion = "15","c"
local smmInside = true
local smmMenuOption = 0
local smmAlphaOptions = {"0.10","0.20","0.30","0.40","0.50","0.60","0.70","0.75","0.80","0.85","0.90","0.95","1.00"}
local smmScaleOptions = {"0.80","0.90","1.00","1.05","1.10","1.15","1.20","1.25","1.30","1.35","1.40","1.50"}
local smmSkins = {
  { square = false,
    texture = "Interface\\Minimap\\UI-Minimap-Border",
    mask = "textures\\MinimapMask",
  },
  { square = false,
    texture = nil,
    mask = "textures\\MinimapMask",
  },
  { square = true,
    texture = "Interface\\AddOns\\simpleMinimap\\skins\\SquareMinimapBorder",
    mask = "Interface\\AddOns\\simpleMinimap\\skins\\SquareMinimapMask",
  },
  { square = true,
    texture = nil,
    mask = "Interface\\AddOns\\simpleMinimap\\skins\\SquareMinimapMask",
  },
  { square = true,
    texture = "Interface\\AddOns\\simpleMinimap\\skins\\dLxBorder",
    mask = "Interface\\AddOns\\simpleMinimap\\skins\\dLxMask",
  },
  { square = true,
    texture = nil,
    mask = "Interface\\AddOns\\simpleMinimap\\skins\\dLxMask",
  },
}
local smmConst = {
  frames = {
    map = MinimapCluster,
    quest = QuestWatchFrame,
    doll = DurabilityFrame,
    timer = QuestTimerFrame,
    capture = smm_captureMover,
  },
  frameDef = {
    map = { anchor = UIParent, point = "TOPRIGHT", rpoint = "TOPRIGHT", x = 0, y = 0 },
    quest = { anchor = MinimapCluster, point = "TOPRIGHT", rpoint = "BOTTOMRIGHT", x = 0, y = 10 },
    doll = { anchor = MinimapCluster, point = "TOPRIGHT", rpoint = "BOTTOMRIGHT", x = 40, y = 15 },
    timer = { anchor = MinimapCluster, point = "TOPRIGHT", rpoint = "BOTTOMRIGHT", x = 10, y = 0 },
  },
  buttons = {
    bgs = MiniMapBattlefieldFrame,
    meet = MiniMapMeetingStoneFrame,
    mail = MiniMapMailFrame,
    time = GameTimeFrame,
    track = MiniMapTrackingFrame,
    zoomin = MinimapZoomIn,
    zoomout = MinimapZoomOut,
  },
}

--[[ initialize ]]--
--
-- debug / info messages
function smm_debug(text)
  if(ChatFrame2) then ChatFrame2:AddMessage(text,0,0.8,1.0) end
end
--
-- default settings
local function smm_setDefaults()
  smmConf = nil
  smmConf = {
    version = smmVersion,
    alpha = 1,
    scale = 1,
    lock = false,
    skin = 1,
    framePos = {
      map = false,
      quest = false,
      doll = false,
      timer = false,
    },
    buttonPos = {
      bgs = 302,
      meet = 189,
      mail = 169,
      time = 137,
      track = 38,
      zoomin = 209,
      zoomout = 235,
    },
    show = {
      location = true,
      movers = false,
      pings = true,
      time = true,
      zoom = true,
    },
  }
end
--
-- init & feedback for the XML load
function smm_onLoad()
  SLASH_SIMPMM1 = "/smm"
  SlashCmdList["SIMPMM"] = smm_slash
  this:RegisterEvent("PLAYER_ENTERING_WORLD")
  this:RegisterEvent("MINIMAP_PING")
  this:RegisterEvent("MINIMAP_UPDATE_ZOOM")
  smm_debug(":: simpleMinimap version "..smmVersion..smmSubVersion.." loaded ::")
end
--
-- on event
function smm_onEvent()
  if(event=="MINIMAP_PING") then
    return(smm_minimapPing(arg1))
  elseif(event=="MINIMAP_UPDATE_ZOOM") then
    local z = Minimap:GetZoom()
    if (GetCVar("minimapZoom") == GetCVar("minimapInsideZoom")) then
      if (z < 3) then 
        Minimap:SetZoom(z + 1)
      else
        Minimap:SetZoom(z - 1)
      end
    else
      z = nil
    end
    if (tonumber(GetCVar("minimapInsideZoom")) == Minimap:GetZoom()) then
      smmInside = true
      MinimapCluster:SetAlpha(1)
    else
      smmInside = false
      MinimapCluster:SetAlpha(smmConf.alpha)
    end
    if(z) then Minimap:SetZoom(z) end
  elseif(event=="PLAYER_ENTERING_WORLD") then
    if(not smmConf or smmConf.version~=smmVersion) then
      smm_debug("smm :: configuration not present or outdated -- resetting")
      smm_setDefaults()
    end
    for _,f in pairs(smmConst.buttons) do
      f:SetMovable(true)
      f:RegisterForDrag("LeftButton")
      f:SetScript("OnDragStart",function() smm_buttonDrag(true) end)
      f:SetScript("OnDragStop",function() smm_buttonDrag(false) end)
    end
    smm_update()
  end
end


--[[ frame events ]]--
--
-- minimap zoom
function smm_minimapZoom(x)
  if(IsShiftKeyDown()) then
    if(not smmInside) then
      local z = MinimapCluster:GetAlpha()
      if(x > 0) then
        if(z < 1) then smmConf.alpha = z + 0.05 else return end
      else
        if(z > 0) then smmConf.alpha = z - 0.05 else return end
      end
      MinimapCluster:SetAlpha(smmConf.alpha)
    end
  else
    if(x > 0) then
      Minimap_ZoomIn()
    else
      Minimap_ZoomOut()
    end
  end
end
--
-- minimap ping
function smm_minimapPing(x)
  if(smmConf.show.pings) then
    if(UnitName(x)==UnitName("player")) then
      MiniMapPing:EnableMouse(false)
    else
      MiniMapPing:EnableMouse(true)
      MiniMapPing:SetScript("OnEnter",function() GameTooltip:SetOwner(this,"ANCHOR_CURSOR") GameTooltip:SetText("ping by |cFFFFFFCC"..UnitName(x)) GameTooltip:Show() end)
      MiniMapPing:SetScript("OnLeave",function() GameTooltip:Hide() end)
    end
  end
end
--
-- frame drag
function smm_frameDrag(x,y)
  if(not y) then y = "map" end
  if(x and not smmConf.lock) then
    smmConst.frames[y].isMoving = true
    smmConst.frames[y]:StartMoving()
  elseif(smmConst.frames[y].isMoving) then
    smmConst.frames[y].isMoving = false
    smmConst.frames[y]:StopMovingOrSizing()
    smmConf.framePos[y] = {}
    smmConf.framePos[y].x,smmConf.framePos[y].y = smmConst.frames[y]:GetCenter()
    smm_update()
  end
end
--
-- button drag
function smm_buttonDrag(x)
  local function getPos()
    local cx,cy = GetCursorPosition(UIParent)
    local mx,my = Minimap:GetLeft(),Minimap:GetBottom()
    local z = Minimap:GetEffectiveScale()
    return(math.deg(math.atan2(cy/z-my-70,mx-cx/z+70)))
  end
  if(x and not smmConf.lock) then
    this.moving = true
    this:SetScript("OnUpdate",function()
      local x,y,z = 0,0,getPos()
      this:ClearAllPoints()
      if(smmSkins[smmConf.skin]["square"]) then
        x = math.max(-81,math.min(110*cos(z),81))
        y = math.max(-81,math.min(110*sin(z),81))
      else
        x = 81*cos(z)
        y = 81*sin(z)
      end
      this:SetPoint("TOPLEFT",Minimap,"TOPLEFT",52-x,y-54)
    end)
    this:StartMoving()
  elseif(this.moving) then
    this.moving = false
    this:StopMovingOrSizing()
    this:SetScript("OnUpdate",nil)
    for n,f in pairs(smmConst.buttons) do
      if(f==this) then smmConf.buttonPos[n] = getPos() end
    end
  end
end
--
-- minimap update
function smm_update()
  MinimapBorder:SetTexture(smmSkins[smmConf.skin]["texture"])
  Minimap:SetMaskTexture(smmSkins[smmConf.skin]["mask"])
  for n,f in pairs(smmConst.frames) do
    if(smmConf.framePos[n]) then
      if(not f.smmTouched) then
        f.smm_ClearAllPoints = f.ClearAllPoints
        f.smm_SetAllPoints = f.SetAllPoints
        f.smm_SetPoint = f.SetPoint
        f.ClearAllPoints = function() end
        f.SetAllPoints = function() end
        f.SetPoint = function() end
        f.smmTouched = true
      end
      f:smm_ClearAllPoints()
      f:smm_SetPoint("CENTER",UIParent,"BOTTOMLEFT",smmConf.framePos[n].x,smmConf.framePos[n].y)
    elseif(f.smmTouched) then
      f.ClearAllPoints = f.smm_ClearAllPoints
      f.SetAllPoints = f.smm_SetAllPoints
      f.SetPoint = f.smm_SetPoint
      f.smm_ClearAllPoints = nil
      f.smm_SetAllPoints = nil
      f.smm_SetPoint = nil
      f.smmTouched = nil
      f:ClearAllPoints()
      f:SetPoint(smmConst.frameDef[n].point,smmConst.frameDef[n].anchor,smmConst.frameDef[n].rpoint,smmConst.frameDef[n].x,smmConst.frameDef[n].y)
    end
  end
  for n,f in pairs(smmConst.buttons) do
    if(smmConf.buttonPos[n]) then
      local x,y = 0,0
      if(smmSkins[smmConf.skin]["square"]) then
        x = math.max(-81,math.min(110*cos(smmConf.buttonPos[n]),81))
        y = math.max(-81,math.min(110*sin(smmConf.buttonPos[n]),81))
      else
        x = 81*cos(smmConf.buttonPos[n])
        y = 81*sin(smmConf.buttonPos[n])
      end
      f:ClearAllPoints()
      f:SetPoint("TOPLEFT",Minimap,"TOPLEFT",52-x,y-54)
    end
  end
  MinimapCluster:SetScale(smmConf.scale)
  if(smmConf.show.location) then
    MinimapZoneTextButton:Show()
    MinimapToggleButton:Show()
    MinimapBorderTop:Show()
    MinimapZoneTextButton:Enable()
    MinimapToggleButton:Enable()
  else
    MinimapZoneTextButton:Disable()
    MinimapToggleButton:Disable()
    MinimapZoneTextButton:Hide()
    MinimapToggleButton:Hide()
    MinimapBorderTop:Hide()
  end
  if(smmConf.show.time) then
    GameTimeFrame:Show()
  else
    GameTimeFrame:Hide()
  end
  if(smmConf.show.zoom) then
    MinimapZoomIn:Show()
    MinimapZoomOut:Show()
  else
    MinimapZoomIn:Hide()
    MinimapZoomOut:Hide()
  end
  if(smmInside) then
    MinimapCluster:SetAlpha(1)
  else
    MinimapCluster:SetAlpha(smmConf.alpha)
  end
  if(smmConf.lock or not smmConf.show.movers) then
    smm_questMover:Hide()
    smm_dollMover:Hide()
    smm_timerMover:Hide()
  else
    smm_questMover:Show()
    smm_dollMover:Show()
    smm_timerMover:Show()
  end
end


--[[ slash command handler ]]--
--
function smm_slash(x)
  local _,_,y,z = string.find(x,"(%w+)%s*(.*)")
  if(y=="hide" and smmConf.hide[z]==false) then
    smmConf.hide[z]=true
    smm_debug("smm :: hiding minimap "..z)
  elseif(y=="show" and smmConf.hide[z]) then
    smmConf.hide[z]=false
    smm_debug("smm :: showing minimap "..z)
  elseif(y=="alpha" and tonumber(z)) then
    z=tonumber(z)
    if (z<0) then z=0 elseif(z>1) then z=1 end
    smmConf.alpha = z
    smm_debug("smm :: setting minimap alpha ("..z..")")
  elseif(y=="scale" and tonumber(z)>0) then
    z=tonumber(z)
    if(smmConf.framePos.map) then
      smmConf.framePos.map.x = (smmConf.scale / z) * smmConf.framePos.map.x
      smmConf.framePos.map.y = (smmConf.scale / z) * smmConf.framePos.map.y
    end
    smmConf.scale = z
    smm_debug("smm :: setting minimap scale ("..z..")")
  elseif(y=="lock") then
    if(smmConf.lock) then
      smmConf.lock = false
      smm_debug("smm :: unlocking minimap")
    else
      smmConf.lock = true
      smm_debug("smm :: locking minimap")
    end
  elseif(y=="skin") then
    smmConf.skin = smmConf.skin + 1
    if(not smmSkins[smmConf.skin]) then smmConf.skin = 1 end
    smm_debug("smm :: changing the minimap skin ("..smmConf.skin..")")
  elseif(y=="ping") then
    if(smmConf.showPings) then
      smmConf.showPings = false
      smm_debug("smm :: turning ping tooltips off")
    else
      smmConf.showPings = true
      smm_debug("smm :: turning ping tooltips on")
    end
  elseif(y=="reset") then
    smm_setDefaults()
    smm_debug("smm :: resetting minimap")
  else
    for _,t in pairs(sm_HELP) do
      DEFAULT_CHAT_FRAME:AddMessage(t,0.9,0.8,0)
    end
    return
  end
  smm_update()
end


--[[ drop-down menu ]]--
--
function smm_dropDown_initialize()
  if(smmConf) then
    if(UIDROPDOWNMENU_MENU_LEVEL==1) then
      UIDropDownMenu_AddButton({text="simpleMinimap |cFFFFFF99v"..smmVersion..smmSubVersion, isTitle=true, textHeight=13})
      UIDropDownMenu_AddButton({text="", notClickable=true, notCheckable=true})
      UIDropDownMenu_AddButton({text="|cFFFFCC66"..smm_SHOW_HEADING, notClickable=true, notCheckable=true, justifyH="CENTER"})
      UIDropDownMenu_AddButton({value=1, text=smm_LOCATION, tooltipText=smm_LOCATION_TOOLTIP, checked=smmConf.show.location, func=smm_dropDown_update, keepShownOnClick=true})
      UIDropDownMenu_AddButton({value=2, text=smm_MOVERS, tooltipText=smm_MOVERS_TOOLTIP, checked=smmConf.show.movers, func=smm_dropDown_update, keepShownOnClick=true})
      UIDropDownMenu_AddButton({value=3, text=smm_PING, tooltipText=smm_PING_TOOLTIP, checked=smmConf.show.pings, func=smm_dropDown_update, keepShownOnClick=true})
      UIDropDownMenu_AddButton({value=4, text=smm_TIME, tooltipText=smm_TIME_TOOLTIP, checked=smmConf.show.time, func=smm_dropDown_update, keepShownOnClick=true})
      UIDropDownMenu_AddButton({value=5, text=smm_ZOOM, tooltipText=smm_ZOOM_TOOLTIP, checked=smmConf.show.zoom, func=smm_dropDown_update, keepShownOnClick=true})
      UIDropDownMenu_AddButton({text="", notClickable=true, notCheckable=true})
      UIDropDownMenu_AddButton({value=10, text=smm_ALPHA, tooltipText=smm_ALPHA_TOOLTIP, notCheckable=true, hasArrow=true, func=smm_dropDown_update, keepShownOnClick=true, justifyH="CENTER"})
      UIDropDownMenu_AddButton({value=11, text=smm_SCALE, tooltipText=smm_SCALE_TOOLTIP, notCheckable=true, hasArrow=true, func=smm_dropDown_update, keepShownOnClick=true, justifyH="CENTER"})
      UIDropDownMenu_AddButton({value=12, text=smm_SKIN, tooltipText=smm_SKIN_TOOLTIP, notCheckable=true, hasArrow=true, func=smm_dropDown_update, keepShownOnClick=true, justifyH="CENTER"})
      UIDropDownMenu_AddButton({text="", notClickable=true, notCheckable=true})
      UIDropDownMenu_AddButton({value=20, text=smm_LOCK, tooltipText=smm_LOCK_TOOLTIP, checked=smmConf.lock, func=smm_dropDown_update, keepShownOnClick=true})
      UIDropDownMenu_AddButton({value=21, text=smm_RESET, tooltipText=smm_RESET_TOOLTIP, func=smm_dropDown_update})
    else
      if(this.value==10) then
        for i in smmAlphaOptions do
          if(tonumber(smmAlphaOptions[i])==tonumber(smmConf.alpha)) then
            UIDropDownMenu_AddButton({text="|cFFFFCC66"..smmAlphaOptions[i], notCheckable=true, notClickable=true, justifyH="CENTER", owner=this.value},2)
          else
            UIDropDownMenu_AddButton({value=i, text=smmAlphaOptions[i], notCheckable=true, func=smm_dropDown_update, justifyH="CENTER", owner=this.value},2)
          end
        end
      elseif(this.value==11) then
        for i in smmScaleOptions do
          if(tonumber(smmScaleOptions[i])==tonumber(smmConf.scale)) then
            UIDropDownMenu_AddButton({text="|cFFFFCC66"..smmScaleOptions[i], notCheckable=true, notClickable=true, justifyH="CENTER", owner=this.value},2)
          else
            UIDropDownMenu_AddButton({value=i, text=smmScaleOptions[i], notCheckable=true, func=smm_dropDown_update, justifyH="CENTER", owner=this.value},2)
          end
        end
      elseif(this.value==12) then
        for i in smmSkins do
          if(i==smmConf.skin) then
            UIDropDownMenu_AddButton({text="|cFFFFCC66"..i, notCheckable=true, notClickable=true, justifyH="CENTER", owner=this.value},2)
          else
            UIDropDownMenu_AddButton({value=i, text=i, notCheckable=true, func=smm_dropDown_update, justifyH="CENTER", owner=this.value},2)
          end
        end
      end
    end
  end
end
--
function smm_dropDown_update()
  local x = this.value
  local y = this.owner
  if(not y) then
    if(x==1) then
      smmConf.show.location = not UIDropDownMenuButton_GetChecked()
    elseif(x==2) then
      smmConf.show.movers = not UIDropDownMenuButton_GetChecked()
    elseif(x==3) then
      smmConf.show.pings = not UIDropDownMenuButton_GetChecked()
    elseif(x==4) then
      smmConf.show.time = not UIDropDownMenuButton_GetChecked()
    elseif(x==5) then
      smmConf.show.zoom = not UIDropDownMenuButton_GetChecked()
    elseif(x==10) then
      ToggleDropDownMenu(2,10,smm_dropDown,this)
      return
    elseif(x==11) then
      ToggleDropDownMenu(2,11,smm_dropDown,this)
      return
    elseif(x==12) then
      ToggleDropDownMenu(2,12,smm_dropDown,this)
      return
    elseif(x==20) then
      smmConf.lock = not UIDropDownMenuButton_GetChecked()
    elseif(x==21) then
      smm_setDefaults()
    else
      return
    end
  elseif(y==10) then
    smmConf.alpha = tonumber(smmAlphaOptions[x])
    smm_debug("smm :: setting minimap alpha ("..smmConf.alpha..")")
  elseif(y==11) then
    local x = tonumber(smmScaleOptions[x])
    if(smmConf.framePos.map) then
      smmConf.framePos.map.x = (smmConf.scale / x) * smmConf.framePos.map.x
      smmConf.framePos.map.y = (smmConf.scale / x) * smmConf.framePos.map.y
    end
    smmConf.scale = x
    smm_debug("smm :: setting minimap scale ("..smmConf.scale..")")
  elseif(y==12) then
    smmConf.skin = x
    smm_debug("smm :: changing the minimap skin ("..x..")")
  else
    return
  end
  smm_update()
end

MinimapCluster:SetMovable(true)
Minimap:RegisterForDrag("LeftButton")
Minimap:SetScript("OnDragStart",function() smm_frameDrag(true,"map") end)
Minimap:SetScript("OnDragStop",function() smm_frameDrag(false,"map") end)
Minimap:SetScript("OnMouseUp",function()
  if(arg1=="RightButton") then
    ToggleDropDownMenu(1,nil,smm_dropDown,"MinimapCluster")
  else Minimap_OnClick() end end)
MinimapZoneTextButton:RegisterForDrag("LeftButton")
MinimapZoneTextButton:SetScript("OnDragStart",function() smm_frameDrag(true,"map") end)
MinimapZoneTextButton:SetScript("OnDragStop",function() smm_frameDrag(false,"map") end)
